/* Скрпит по чтению данных весов МИ ТВ–003/05Н
 * Период опроса 1000 мсек
 * Параметры работы скрипта задаются через аргументы. К примеру: Addr=10, Crc=0
 * Где:
 * Addr=/10 - сетевой адрес весов от 1 до 127. По умолчанию адрес равен 1; (Необходимо установить равным, установленному в весах)
 * Crc = 0/1 - выключить/включить контроль CRC. По умолчанию выключен; (Необходимо установить равным, установленному в весах);
 */


#include <tracker>
#include <args>
#include <time>
#include <string>
#include <serial>

#pragma dynamic               256

#define PERIOD                1000  

#define CMD_GET_NET_WEIGHT    0xC2
#define CMD_GET_GROSS_WEIGHT  0xC3

#define  NET_WEIGHT           0
#define  GROSS_WEIGHT         1

new Addr = 1;    // Сетевой адрес
new UseCRC = 0;  // Используеться ли CRC

#define CRC8_TABLE_SIZE  256

new table_crc[CRC8_TABLE_SIZE]

crc8_generate_table(table[CRC8_TABLE_SIZE], poly)
{
    new msbit = 0x80;
    table[0] = 0;
 
    new t = msbit;
    for (new i = 1; i < CRC8_TABLE_SIZE; i *= 2)
    {
        t = ((t << 1) ^ (t & msbit ? poly: 0)) & 0xFF;
        for (new j = 0; j < i; ++j)
            table[i+j] = (table[j] ^ t) & 0xFF;
    }
}

crc8(table[CRC8_TABLE_SIZE],  data[], nbytes, crc)
{
    new pdata = 0;
    while (nbytes-- > 0)
        crc = (table[(crc ^ data[pdata++]) & 0xff]) & 0xFF;
 
    return crc;
}


/* Формирование запроса */ 
static LibraGenRequest(addr, useCrc, req[], reqLen, buff[64])
{
    new pBuff = 0;
    new crc = 0;
    new InterimReq[64];
    new pIReq = 0;
    
    /* Формируем запрос */
    InterimReq[pIReq++] = addr;
    for(new i = 0; i < reqLen; i++) InterimReq[pIReq++] = req[i]
    /* Считаем CRC */
    if (useCrc)
    {
        crc = crc8(table_crc, InterimReq, pIReq, crc);
        InterimReq[pIReq++] = (crc & 0xFF);
    }
    /* Перекладываем данные в буфер*/
    buff[pBuff++] = 0xFF; // Стартовый символ
    for (new i = 0; i < pIReq; i++)
    {
        buff[pBuff++] = InterimReq[i];
        if (InterimReq[i] == 0xFF) buff[pBuff++] = 0xFE;   
    }
    buff[pBuff++] = 0xFF; // Стоп 2 раза
    buff[pBuff++] = 0xFF;
    return pBuff;
}

/* Получение ответа из принятого буфера и его проверка */
static LibraParseAnswer(buff[], len, addr, Answer[], AnswerBuffSize = sizeof(Answer))
{
    new InterimAnsw[64];
    new SizeAnswer = 0;
    new i = 0;
    
    if (len < 4) return 0; // Минимально возможный размер ответа
    if ((buff[0] != 0xFF) || (buff[1] != addr)) return 0; // Если нет стартового символа или адрес не совпал
    /* Выдергиваем данные из принятото буфера */
    i = 2    
    while (i < len)
    {
        if (buff[i] != 0xFF) 
        {
            InterimAnsw[SizeAnswer++] = buff[i]
            i++;
        }
        else
        {
            if (buff[i+1] == 0xFF) break; // Если два байта FF подряд
            InterimAnsw[SizeAnswer++] = buff[i];
            i += 2;
        }
    }    
    /* Копируем из промежуточного буфера */
    for(i = 0; ((i < SizeAnswer) && (i < AnswerBuffSize)); i++) Answer[i] = InterimAnsw[i];
    return i;    
}

/* Получение данных */
/* Читаем вес НЕТТО*/ 
static bool:LibraReadValueWeight(addr, useCrc, &Float:value, &bool:stable, type = NET_WEIGHT)
{
    new Buff[64];
    new Len;
    new Req[1];
    new iValue;
    new Pow;
    
    if (type == NET_WEIGHT) Req[0] = CMD_GET_NET_WEIGHT; else Req[0] = CMD_GET_GROSS_WEIGHT;
    Len = LibraGenRequest(addr, useCrc, Req, 1, Buff); // Формируем запрос
    rssend(Buff, Len); // Передаем его
    Len = rsrecv(Buff, sizeof(Buff), 500, 100); // Читаем ответ
    if (LibraParseAnswer(Buff, Len, addr, Buff) == 0) return false; // Выделяем полезные данные
    if (Buff[0] != Req[0]) return false; // Если не совпал опкод запроса
    iValue =  (((Buff[3] >> 4) & 0x0F) * 100000)
    iValue += (((Buff[3] >> 0) & 0x0F) * 10000)   
    iValue += (((Buff[2] >> 4) & 0x0F) * 1000)
    iValue += (((Buff[2] >> 0) & 0x0F) * 100) 
    iValue += (((Buff[1] >> 4) & 0x0F) * 10)
    iValue += (((Buff[1] >> 0) & 0x0F) * 1)     
    if (Buff[4] & 0x80) iValue = -iValue;
    if (Buff[4] & 0x10) stable = true; else stable = false;
    Pow = Buff[4] & 0x07;
    value = iValue
    for (new i = 0; i < Pow; i++) value = (value / 10.0);
    return true;
}

@setup()
{
    crc8_generate_table(table_crc, 0x69)
    /* Аргументы работы скрипта */
    argvalue(0, "Addr", Addr);   // Читаем сетевой адрес
    argvalue(0, "Crc", UseCRC);  // Читаем дельту
    printf("Read arg. Addr=%d,UseCrc=%d", Addr, UseCRC);
    /* Проверяем адресс */
    if (Addr < 1) 
    {
        Addr = 1;
    }
    else if (Addr > 127)
    {
        Addr = 1;   
    }  
    /* Если параметры адреса заданы корректно */ 
    if (Addr != 0) settimer(PERIOD) // Запускаем таймер с заданным периодом
    else printf("Error!!!. Incorrect address and serial number"); // Выводим сообщение о ошибке

}

@timer()
{
    new Float:fValue = 0.0;
    new bool:Valid;
    new bool:Stable
    new Value
    
    if (rsopen() == true) /* Удалось открыть порт */
    {  
        printf("Cycle");   
        for (new i = 0; i < 2; i++)
        {
            Valid = LibraReadValueWeight(Addr, UseCRC, fValue, Stable, NET_WEIGHT + i);
            Value = floatround((fValue * 10), floatround_floor); 
            settag((i*2), Value, Valid, 1);
            settag(((i*2)+1), Stable, Valid, 0);             
        }
        rsclose();
    }  
}


